﻿using System.IO;
using System;
using System.Runtime.Serialization;

namespace Contracts
{
    public interface IFilter
    {
        void Start(IStorage s);

        // @p probe of file 
        // Returns probability that this filter can read this file 0..1
        double Feel( FileProbe p);

        bool Load( FileProbe p, IStorage s);

    }

    [Serializable]
    public class FileProbe:ISerializable
    {
        public string Name; // file name without extension
        public string Ext; // file extension in lower case, e.g. "dat"
        public string NameExt; // file name 
        public string Path; // full file path
        public string RelativePath; // full relative file path
        public string Dir; // file dir with ending \\
        public string DirName; // file dir name at top level. eg for c:\data\sample5 it is "sample5"

        public byte[] First2K; // first 2048 bytes of file data. 
        public string FirstLines; // ascii presentation of first2k. If file contains \0, firstlines is cutted to that first \0.

        public StreamReader f;    /// text file reader
        public FileStream fbin;

        public long FileSize; // file size in bytes

        private string _filePath;

        public bool GuiAllowed; // is gui interacion allowed. if false - we work in batch mode so no gui please.

        public DateTime time; // last write time

        /// bin file reader
        /// 
        public FileProbe() { }
        public FileProbe(string filepath) { Generate(filepath); }
        //~FileProbe() { CloseHandles();  }

        public void Generate(string filepath)
        {
            filepath = filepath.ToLower();
            _filePath = filepath;
            var fInfo = new FileInfo(filepath);
            var p = this;
            p.Ext = fInfo.Extension.Length > 0 ? fInfo.Extension.Substring(1) : "";
            p.Name = System.IO.Path.GetFileNameWithoutExtension(filepath);
            p.NameExt = System.IO.Path.GetFileName(filepath);
            p.Path = System.IO.Path.GetFullPath(filepath).ToLower();
            p.Dir = System.IO.Path.GetDirectoryName(p.Path).ToLower() + "\\";
            p.DirName = System.IO.Path.GetFileName(System.IO.Path.GetDirectoryName(p.Path)).ToLower();
            p.f = File.OpenText(p.Path);
            p.fbin = File.OpenRead(p.Path);

            p.First2K = new byte[2048];
            p.fbin.Read(p.First2K, 0, 2048);
            p.fbin.Seek(0, 0);

            p.FileSize = p.fbin.Length;

            p.FirstLines = System.Text.Encoding.ASCII.GetString(p.First2K);

            p.time = File.GetLastWriteTime(p.Path);

            Uri uri1 = new Uri(p.Path);
            Uri uri2 = new Uri(Environment.CurrentDirectory+"\\");
            Uri relativeUri = uri2.MakeRelativeUri(uri1);
            p.RelativePath = Uri.UnescapeDataString(relativeUri.ToString()).Replace("/", "\\").ToLower();

            p.GuiAllowed = true;
        }

        public void CloseHandles()
        {
            f.Close();
            fbin.Close();
        }

        protected FileProbe(SerializationInfo info, StreamingContext context)
        {
           Generate(info.GetString("path"));
        }

        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("path",_filePath);
        }
    }

}
