using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Media;
using System.Windows.Media.Media3D;
using System.Windows;

namespace Contracts
{
    public delegate void OnChanged(object sender);
    public interface IModel3DWithMetaData : IModelPropertyAccessor,IObjectWithMetaData
    {
        /// <summary>
        ///     
        /// </summary>
        /// <typeparam name="T"> </typeparam>
        /// <param name="propertyName"> </param>
        /// <param name="modelPropertyAccessor">,     </param>
        void AddProperty<T>(String propertyName, IModelPropertyAccessor modelPropertyAccessor);
        /// <summary>
        ///   
        /// </summary>
        /// <param name="propertyName"></param>
        void RemoveProperty(String propertyName);
        /// <summary>
        ///     
        /// </summary>
        /// <returns></returns>
        IEnumerable<String> GetProperties();
        /// <summary>
        ///     
        /// </summary>
        /// <param name="propertyName"></param>
        /// <param name="control"> </param>
        /// <param name="title">  </param>
        void SetControl(String propertyName, UIControl control, String title = "");
        /// <summary>
        ///      
        /// </summary>
        /// <param name="propertyName"></param>
        /// <returns> , null -    </returns>
        UIControl GetControl(String propertyName);
        /// <summary>
        ///   
        /// </summary>
        /// <param name="propertyName"></param>
        /// <returns> </returns>
        Type GetPropertyType(String propertyName);

        /// <summary>
        /// Refreshes model on screen. Required after geometry or other properties change
        /// </summary>
        void Refresh();

        /// <summary>
        /// Refreshes the model, if NeedRefresh is true.
        /// NeedRefresh is changed to true when geometry changed via AddNode/AddTriangle
        /// </summary>
        bool RefreshIfNeeded();

        ModelVisual3D Model { get; }
        MeshGeometry3D Mesh { get; set; }
        SolidColorBrush Color { get; set; }
        string ColorStr { set; }
        SolidColorBrush BackColor { get; set; }
        string BackColorStr { set; }
        /// <summary>
        ///  
        /// </summary>
        double Opacity { get; set; }
        /// <summary>
        ///  
        /// </summary>
        bool Visible { get; set; }
        /// <summary>
        ///   
        /// </summary>
        bool Outline { get; set; }
        /// <summary>
        ///   
        /// </summary>
        string Title { get; set; }

        /// <summary>
        /// Adding a node (base point) given by its coordinates
        /// </summary>
        /// <param name="x">the x-coordinate of the node</param>
        /// <param name="y">the y-coordinate of the node</param>
        /// <param name="z">the z-coordinate of the node</param>
        /// <returns>The index of the point in the node storage (can be used to define a triangle vertex)</returns>
        int AddNode(double x, double y, double z);

        /// <summary>
        /// Adding a node (base point) given as a 3D point
        /// </summary>
        /// <param name="p">the 3D point of the node</param>
        /// <returns>The index of the point in the node storage (can be used to define a triangle vertex)</returns>
        int AddNode(Point3D A);

        /// <summary>
        /// Procedure for adding a spearate triangle, which is not a neighbour for any other triangle.
        /// It has status Trusted
        /// </summary>
        /// <param name="A">The first vertex of the triangle</param>
        /// <param name="B">The second vertex of the triangle</param>
        /// <param name="C">The third vertex of the triangle</param>
        void AddTriangleSeparate(Point3D A, Point3D B, Point3D C);

        /// <summary>
        /// Procedure for adding a spearate triangle, which is not a neighbour for any other triangle.
        /// It has status Trusted
        /// </summary>
        /// <param name="xA">The x-coordinate of the first vertex</param>
        /// <param name="yA">The y-coordinate of the first vertex</param>
        /// <param name="zA">The z-coordinate of the first vertex</param>
        /// <param name="xB">The x-coordinate of the second vertex</param>
        /// <param name="yB">The y-coordinate of the second vertex</param>
        /// <param name="zB">The z-coordinate of the second vertex</param>
        /// <param name="xC">The x-coordinate of the third vertex</param>
        /// <param name="yC">The y-coordinate of the third vertex</param>
        /// <param name="zC">The z-coordinate of the third vertex</param>
        void AddTriangleSeparate(double xA, double yA, double zA,
                                  double xB, double yB, double zB,
                                  double xC, double yC, double zC);

        /// <summary>
        /// Procedure for adding a triangle with unknown orientation
        /// </summary>
        /// <param name="A">the index of the first vertex</param>
        /// <param name="B">the index of the second vertex</param>
        /// <param name="C">the index of the third vertex</param>
        void AddTriangle(int A, int B, int C);

        /// <summary>
        /// Procedure for adding a triangle with fixed orientation
        /// </summary>
        /// <param name="A">the index of the first vertex</param>
        /// <param name="B">the index of the second vertex</param>
        /// <param name="C">the index of the third vertex</param>
        void AddTriangleTrusted(int A, int B, int C);

        ///// <summary>
        ///// Procedure finalizing construction of the mesh: all triangles are added, let's orient them
        ///// </summary>
        ///// <exception cref="OrientationException">Thrown when the mesh cannot be oriented 
        ///// (that is, some neighbour triangles should have outer normals with angle larger than 90 degrees
        ///// according to some Trusted triangles)</exception>
        //void FinalizeConstruction();

        /// <summary>
        /// Add a sphere to the object
        /// </summary>
        /// <param name="c">The center of the sphere</param>
        /// <param name="r">The radius of the sphere</param>
        /// <param name="meridians">Number of meridians in the approximation grid</param>
        /// <param name="parallels">Number of parallels in the approximation grid</param>
        void AddSphere(Point3D c, double r, int meridians = 24, int parallels = 12);

        /// <summary>
        /// Add a sphere to the object
        /// </summary>
        /// <param name="x">The x-coordinate of the center</param>
        /// <param name="y">The y-coordinate of the center</param>
        /// <param name="z">The z-coordinate of the center</param>
        /// <param name="r">The radius of the sphere</param>
        /// <param name="meridians">Number of meridians in the approximation grid</param>
        /// <param name="parallels">Number of parallels in the approximation grid</param>
        void AddSphere(double x, double y, double z, double r, int meridians = 24, int parallels = 12);

        /// <summary>
        /// Add a point to the scene. A point is represented by a shpere of a small radius
        /// </summary>
        /// <param name="p">Position of the point</param>
        /// <param name="thickness">Thickness of the poin (the radius of the sphere)</param>
        void AddPoint(Point3D p, double thickness = 0.01);

        /// <summary>
        /// Add a point to the scene. A point is represented by a shpere of a small radius
        /// </summary>
        /// <param name="x">The x-coordinate of the point</param>
        /// <param name="y">The y-coordinate of the point</param>
        /// <param name="z">The z-coordinate of the point</param>
        /// <param name="thickness">Thickness of the poin (the radius of the sphere)</param>
        void AddPoint(double x, double y, double z, double thickness = 0.01);

        /// <summary>
        /// Add a truncated cone
        /// </summary>
        /// <param name="p1">The center of the first base</param>
        /// <param name="r1">The radius of the first base</param>
        /// <param name="p2">The center of the second base</param>
        /// <param name="r2">The radius of the second base</param>
        /// <param name="addBases">Whether to close the bases</param>
        /// <param name="meridians">Number of meridians</param>
        void AddTruncCone(Point3D p1, double r1, Point3D p2, double r2, bool addBases = false, int meridians = 20);

        /// <summary>
        /// Add a cone
        /// </summary>
        /// <param name="p1">The top of the cone</param>
        /// <param name="p2">The center of the base</param>
        /// <param name="r2">The radius of the base</param>
        /// <param name="addBase">Whether to close the base</param>
        /// <param name="meridians">Number of meridians</param>
        void AddCone(Point3D p1, Point3D p2, double r2, bool addBase = false, int meridians = 20);

        /// <summary>
        /// Add a cone
        /// </summary>
        /// <param name="p1">The center of the first base</param>
        /// <param name="p2">The center of the second base</param>
        /// <param name="r">The radius of the cylinder</param>
        /// <param name="addBases">Whether to close the bases</param>
        /// <param name="meridians">Number of meridians</param>
        void AddCylinder(Point3D p1, Point3D p2, double r, bool addBases = false, int meridians = 20);

        /// <summary>
        /// Add a segment netween two points as a "thin" cylinder
        /// </summary>
        /// <param name="p1">The first point</param>
        /// <param name="p2">The second point</param>
        /// <param name="thickness">Thickness of the line (radius of the cylinder)</param>
        /// <param name="meridians">Number of meridians</param>
        void AddLine(Point3D p1, Point3D p2, double thickness = 0.01, int meridians = 8);

        /// <summary>
        /// Add a polygonal line as a number of its edges
        /// </summary>
        /// <param name="p">Array of polyline vertices</param>
        /// <param name="thickness">Thickness of the line (radius of the cylinders)</param>
        /// <param name="isClosed">Whether the line is closed (to link the last and the first points)</param>
        /// <param name="meridians">Number of meridians</param>
        void AddPolyline(IEnumerable<Point3D> p, double thickness = 0.01, bool isClosed = false, int meridians = 8);

        void AddBox(Point3D center, double xLength, double yLength, double zLength);

        void AddPlane(Point3D center, double sideLength, double height);

        void AddPyramid(Point3D center, double sideLength, double height);

        /// <summary>
        /// Bare UI elements
        /// </summary>
        /// <param name="element"></param>
        /// <param name="text"></param>
        /// <param name="isTextOnTop"></param>
        void AddUIElement(FrameworkElement element, String text = "", Boolean isTextOnTop = true);

        IEnumerable<IUIElementWithMetaData> GetUIElements();

        /// <summary>
        ///     
        /// </summary>
        /// <param name="geometry"></param>
        void Reset(bool geometry = true, bool ui_elements = false);

        /// <summary>
        ///       
        /// </summary>
        double ColorValue  { get; set; }

        /// <summary>
        ///  hit-value    ;    hit-value 
        /// </summary>
        int HitValue { get; set; }

        /// <summary>
        ///    NodeHitValue;  hit-value    ;    hit-value 
        /// </summary>
        Int64 HitValue2 { get; set; }

        Vector3D Unscalable { get; set; }
        ScaleTransform3D ScaleTransform { get; set; }

        event PropertyChangedEventHandler HitTested;

        /*/// <summary>
        ///    hit-value 
        /// </summary>
        int UserHitValue { get; set; }
        */

        /// <summary>
        ///  Storage 
        /// </summary>
        /// <returns></returns>
        IStorage GetStorage();

        /// <summary>
        ///  Storage ,  Storage();
        /// </summary>
        /// <returns></returns>
        IStorage GetScene(); // 

        event ScaleChanged OnScaleChanged;
    }
}