﻿using System;
using System.Collections.Generic;
using System.Windows.Media.Media3D;

namespace Contracts
{
    public interface IStorage : ICameraTransformationController
    {
        void SetState(string stateMessage);
        IModelGroup AddModelGroup(String name, IModelGroup parent = null, AddModeIfExists mode = AddModeIfExists.Replace);
        IModel3DHolder AddModel3DHolder(IModel3DWithMetaData model, IModelGroup parent = null);

        IModel3DWithMetaData AddModel(string name, string title = null, bool visible = true, IModelGroup group = null,
                                      AddModeIfExists mode = AddModeIfExists.ReturnOldCleared);

        ILightWithMetaData AddPointLight(string name, Point3D position, string title = null, bool visible = true,
                                         AddModeIfExists mode = AddModeIfExists.Replace);

        ILightWithMetaData AddDirectionalLight(string name, Vector3D direction, string title = null, bool visible = true,
                                               AddModeIfExists mode = AddModeIfExists.Replace);

        ILightWithMetaData AddAmbientLight(string name, string title = null, bool visible = true,
                                           AddModeIfExists mode = AddModeIfExists.Replace);

        ICameraWithMetaData AddPerspectiveCamera(string name, Point3D position, Vector3D look, Vector3D up,
                                                 double fieldOfView, string title = null,
                                                 AddModeIfExists mode = AddModeIfExists.Replace,
                                                 bool rotationEnabled = true, bool zoomEnabled = true,
                                                 bool moveInPerpendicularPlaneEnabled = true,
                                                 bool moveInParallelPlaneEnabled = true);

        ICameraWithMetaData AddOrthographicCamera(string name, Point3D position, Vector3D look, Vector3D up,
                                                  double width, string title = null,
                                                  AddModeIfExists mode = AddModeIfExists.Replace,
                                                  bool rotationEnabled = true, bool zoomEnabled = true,
                                                  bool moveInPerpendicularPlaneEnabled = true,
                                                  bool moveInParallelPlaneEnabled = true);

        IModel3DWithMetaData GetModelByName(string name);
        ICameraWithMetaData GetCameraByName(string name);
        ILightWithMetaData GetLightByName(string name);
        void RemoveModelByName(string name);
        void RemoveCameraByName(string name);
        void RemoveLightByName(string name);
        void RemoveModel(IModel3DWithMetaData model);
        void RemoveCamera(ICameraWithMetaData camera);
        void RemoveLight(ILightWithMetaData light);
        IList<IModelGroup> GetModelGroupByName(string name, IModelGroup root = null);
        IList<IModel3DHolder> GetModelHolders(IModel3DWithMetaData model3DWithMetaData, IModelGroup root = null);
        IList<IModel3DWithMetaData> GetAllModels(IModelGroup root = null);
        void RemoveModelHolders(IModel3DWithMetaData model, IModelTreeNode root = null, bool removeEmptyGroups = true);
        IEnumerable<IUIControlFactory> GetControlFactoryByType(Type type);
        IUIControlFactory GetControlFactoryByName(Type type);
        void AddRawData(string name, object obj);
        Object GetRawData(string name);
        void CommitRawData(string name);
        void RemoveRawData(string name);

        bool LoadData(string filepath, bool allow_interactive = false);

        int RefreshUpdatedModels();

        void MakeSelectedModel(string name);
        ICameraWithMetaData SelectedCamera { get; set; }
        IModelPropertyAccessor GetModelGroupPropertyAccessor(IModelGroup modelGroup);
        IModel3DHolder GetModelHolderFromTreeByName(string name, IModelGroup root = null);
        IModelTreeNode GetNodeByFullPath(string fullPath, IModelTreeNode root = null);
        void ResetAndInit();
        bool LoadFromFile(string fileName);

        /// <summary>
        /// Зум сцены накапливаемый
        /// </summary>
        /// <param name="k"></param>
        void Zoom(double k);

        void Rotate(double x, double y, double z);
        void ZoomScene(Vector3D zoom, bool relative = true);
        void ZoomScene(double zoom, bool relative = true);
        void RotateScene(Vector3D axis, double angle, Point3D center = default(Point3D), bool relative = true);
    }

    public delegate void SelectedCameraChangeHandler(ICameraWithMetaData oldCamera, ICameraWithMetaData newCamera);

    public enum AddModeIfExists
    {
        Replace,
        ReturnOld,
        ReturnOldCleared,
        ThrowException
    }

    public enum ListType
    {
        Models,
        Lights,
        Cameras,
        Unknown
    }

    public class ElementExistException : Exception
    {
    };
}