﻿using System;

namespace Contracts
{
    /// <summary>
    /// Тип, представляющий вещественный промежуток от Min до Max
    ///  с выбранным значением на данном промежутке
    /// </summary>
    [Serializable]
    public class Range:IAbleToGroup
    {
        private double? _value;
        /// <summary>
        /// Левая граница промежутка
        /// </summary>
        public double Min { get; private set; }
        /// <summary>
        /// Правая граница промежутка
        /// </summary>
        public double Max { get; private set; }

        /// <summary>
        /// Выбранное значение, null - если не определено
        /// </summary>
        public double? Value
        {
            get
            {
                return _value;
            }
            set
            {
                if (value!=null && (value > Max || value<Min)) throw new ArgumentOutOfRangeException("Value");
                _value = value;
            }
        }
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="value">Выбранное значение, null - если не определено</param>
        /// <param name="min">Минимальное значение</param>
        /// <param name="max">Максимальное значение</param>
        public Range(double? value, double min, double max)
        {
            Min = min;
            Max = max;
            if (max<min) throw new ArgumentOutOfRangeException("Max");
            Value = value;
        }

        /// <summary>
        /// Пересечение промежутков
        /// </summary>
        /// <param name="another">Другой промежуток</param>
        /// <returns>Результирующий промежуток, null - если промежутки не пересекаются</returns>
        public IAbleToGroup GroupWith(IAbleToGroup another)
        {
            if (!(another is Range)) return null;
            var anotherRange = (Range) another;
            var newMin = Min < anotherRange.Min ? anotherRange.Min : Min;
            var newMax = Max > anotherRange.Max ? anotherRange.Max : Max;
            var value = Value == anotherRange.Value ? Value : null;
            return newMin > newMax ? null : new Range(value, newMin, newMax);
        }

        public override string ToString()
        {
            return _value.ToString();
        }
    }
}
