require 'rubygems'
require 'restclient'

# добавим в каталог либ - текущий каталог, это надо чтобы успешно подключать файлы из него
$LOAD_PATH.unshift File.expand_path(File.dirname(__FILE__))
require 'cobravsmongoose'

module Evented

  #SERVER_URL = "http://localhost:5000"
  SERVER_URL = "http://svn.lact.ru:5000"
  PUSH_MODE = :logged
  # :normal - обычный нормальный синхронный режим.
  # :threaded - асинхронный режим с посылкой в параллельном потоке.
  # :logged - асинхронный режим с запуском демона.
  # :local_print - печать и все.
  # :local_log - запись в локальный лог и все.
  # :nop - ничегошеньги не делать

  LOGS_DIR = ENV["EVENTED_LOGS_DIR"] ? ENV["EVENTED_LOGS_DIR"] : (defined?(RAILS_ROOT) ? File.join(RAILS_ROOT,"log") : ".")
  EVENTS_LOG = "event_pusher.log"   
  DAEMON_LOG = "evented_daemon.log"
  DAEMON_LOCK = "evented_daemon.lock"

  EVENTS_PREFIX = ""

  CODE_VERSION = File.mtime( File.join( File.expand_path(File.dirname(__FILE__)),"evented_daemon.rb") ).to_i
  WINRUN = (RUBY_PLATFORM =~ /mswin|mingw/) ? true : false
  RUBY_CALL = WINRUN ? "rubyw.exe" : "ruby"

#  CODE_VERSION = ["evented_daemon.rb","event_pusher.rb"].map{ |f| File.mtime( File.join( File.expand_path(File.dirname(__FILE__)),f) ).strftime("#{f}=%Y%m%d-%H%M%S") }.join("_")

  def self.push(event_name, event_value = 1, options = {})

    if event_name.class != String then event_name = event_name.to_s end
    event_name.gsub!("\n", "")
    if (event_value.class == String) then event_value.gsub!("\n", "") end

    event_date = options[:date] || Time.now 
    server = options[:server] || SERVER_URL
    event_date_utc_str = event_date.utc.strftime("%Y/%m/%d %H:%M:%S UTC")

    if PUSH_MODE == :normal
      self.perform_network_push(event_name, event_value, event_date_utc_str, server)
    elsif PUSH_MODE == :threaded
      # puts "performing threaded push"
      Thread.new { self.perform_network_push(event_name, event_value, event_date_utc_str, server) }
    elsif PUSH_MODE == :nop
    elsif PUSH_MODE == :local_print
      # puts "performing print push"
      puts "Evented.push: event=#{event_name} val=#{event_value} time=#{event_date_utc_str}"
    elsif PUSH_MODE == :logged or PUSH_MODE == :local_log
      params_hash = { "event" => {
          "name" => {"$" => (EVENTS_PREFIX+event_name)},
          "value" => {"$" => event_value.to_s},
          "date" =>   {"$" => event_date_utc_str},
          "server" => {"$" => server},
          "code_version" => {"$" => CODE_VERSION.to_s} }
      }
      log_line_badger_fish = CobraVsMongoose.hash_to_xml(params_hash)

      File.open(File.join(LOGS_DIR, EVENTS_LOG), 'a') {|f| f.puts(log_line_badger_fish) }

      if PUSH_MODE == :logged
        df = File.open( File.join( LOGS_DIR, DAEMON_LOCK ),"a+" ) #FIXED: тут было File.join( ".", DAEMON_LOCK ) что неверно, мы не обязательно в "." работаем
        # проверяем доступ к файлу. Если можем лочить - значит скорее всего фоновая прога еще не запущена
        if df.flock(File::LOCK_EX|File::LOCK_NB) == 0
          df.flock(File::LOCK_UN)
          df.close
          ENV["EVENTED_LOGS_DIR"] = LOGS_DIR # передаем фоновой проге информацию о том, где живут логи
          daemon_script = File.join File.expand_path(File.dirname(__FILE__)), "evented_daemon.rb" 
          Evented.system_call_with_no_block("#{RUBY_CALL} #{daemon_script}")
        end
      end

    else
      puts "Evented.push failed - unknown mode (Evented::PUSH_MODE=#{PUSH_MODE})"
    end
  end

  def self.perform_network_push(event_name, event_value, event_date_utc_str, server)
    # event_name - тип произошедшего события (строка)
    # event_value - значение, соответствующее данному событию (число или строка)
    # event_date_utc_str - дата произошедшего события (строка-время в UTC))
    begin
      puts "Evented.perform_network_push is pushing event #{event_name} with val #{event_value} and date #{event_date_utc_str} to #{server}"
      RestClient.post "%s/graphs/%s" % [server, CGI::escape(EVENTS_PREFIX+event_name)], :value => event_value, :timestamp => event_date_utc_str
      # add .%L - for milliseconds
      puts "success!"
      return 0
    rescue => e
      puts "Evented.perform_network_push got some other error: #{e.message}"
      return 1
    end
  end

  def self.delete(event_name)
    # event_name - тип события, данные о котором нужно удалить

    RestClient.delete "%s/graphs/%s" % [SERVER_URL, CGI::escape(EVENTS_PREFIX+event_name)]
  end

  if defined?(RAILS_ROOT)
  # send stats about all models in project
  def self.rails_daily # TODO: Pavel: add explicit models list
#     Evented::SERVER_URL = "http://localhost:5000"
#     Evented::PUSH_MODE = :print

    # http://stackoverflow.com/questions/1233692/get-a-list-of-all-models-from-rails
    Dir.glob(RAILS_ROOT + '/app/models/*.rb').each { |file| require_dependency(File.basename(file)) rescue nil }
    models = Object.subclasses_of(ActiveRecord::Base)
    models.each do |model|
      begin
        key = "daily"
        key << "/#{model.superclass}" if model.superclass != ActiveRecord::Base
        key << "/#{model.name}"
        # records count
        Evented.push "#{key}/total",model.size
        # scopes count
        for sco in model.scopes.keys do
          Evented.push "#{key}/#{sco}",Site.send(sco.to_s).size.to_s
        end
      rescue
      end
    end
  end # rails_daily

  end # if defined?(RAILS_ROOT)


  def self.system_call_with_no_block(s)
    puts "**************** INVOKING EXTERNAL COMMAND #{s}, WINRUN IS #{WINRUN}"
    if WINRUN
      if system("psexec -d #{s}")
        puts "Executed using PSEXEC"
#      elsif system("start cmd /k #{s}") # DEBUG MODE
#        puts "Executed using START cmd /k (inherits descriptors (?) - bad)"
#      elsif system("start cmd /c #{s}")
#        puts "Executed using START cmd /c (inherits descriptors (?) - bad)"
      elsif system("start #{s}")
        puts "Executed using START (inherits descriptors (?) - bad)"
      else
        puts "Execute failed!"
      end
    else
      puts "Executing in Linux style via fork-exec"
      Process.detach fork{exec(s)}
    end
  end

end


  


