# добавим в каталог либ - текущий каталог, это надо чтобы успешно подключить event_pusher.rb
$LOAD_PATH.unshift File.expand_path(File.dirname(__FILE__))

require 'rubygems'
require 'event_pusher'
require 'time'
require "cobravsmongoose"

IDLE_TIMEOUT = 2 # minutes; таймаут происходит тогда, когда лог событий пустует в течение указанного значения или более

@events_posted_total = 0
@idle = false
@idle_start = Time.now
@events_log_path = File.join( Evented::LOGS_DIR,Evented::EVENTS_LOG)

# TODO: бывает что демон выключится по code_version, а запуститься заново не успеет - скрипт-пользователь отработал. На заметку.
# TODO: по уму бы надо еще отлавливать SIGналы, и разлачивать файл. Или он сам разлочится?...

def manage_events_log
  if File.exists?(@events_log_path) & !File.zero?(@events_log_path)
    begin
      @idle = false

      post_result = try_to_post_events
      last_succeed_idx = post_result[:last_succeed_idx]
      out_of_date = post_result[:out_of_date]
      @events_posted_total = @events_posted_total + last_succeed_idx + 1

      f = File.open(@events_log_path, "r")
      text = f.readlines
      f.close

      File.open(@events_log_path, "w") do |f|
        f.puts(text[(last_succeed_idx + 1) .. text.count-1])   # переписываем в файл те события, которые не удалось запостить
      end

      log "File #{@events_log_path} was managed at #{Time.now}.\nNumber of events posted: #{last_succeed_idx+1}.\n\n"

      if out_of_date
        terminate_self("code version is out of date")
      end

    rescue => e
      log "manage_events_log: I got exception" 
      log e.message
    end
  else
    if @idle == false then @idle_start = Time.now end
    @idle = true
    puts "Nothing to manage. Idle for #{(Time.now - @idle_start).to_i} seconds."
  end
end

def try_to_post_events
  last_succeed_idx = -1
  out_of_date = false

  File.open(@events_log_path, "r") do |f|        # Пытаемся запостить из файла событий
    f.each_line do |line|
      event_data = CobraVsMongoose.xml_to_hash(line)["event"]

      if (event_data["code_version"]["$"].to_i > Evented::CODE_VERSION)
        failed = 1
        out_of_date = true
        log "Detected code version change: eventlog #{event_data['code_version']["$"]} > code #{Evented::CODE_VERSION}"
      else
        failed = Evented.perform_network_push(event_data["name"]["$"], event_data["value"]["$"], event_data["date"]["$"], event_data["server"]["$"])
      end
      failed == 1 ? break : last_succeed_idx = last_succeed_idx + 1
    end
  end

  return {:last_succeed_idx => last_succeed_idx, :out_of_date => out_of_date}
end

def terminate_self(reason)
  log(%Q[
    Evented-daemon version #{Evented::CODE_VERSION} has been stopped at #{Time.now}.\n
    Reason: #{reason}.\n
    Total number of events posted: #{@events_posted_total}.\n
    #####################\n\n])

  exit
end

def log(msg)
  puts msg
  File.open( File.join( Evented::LOGS_DIR,Evented::DAEMON_LOG), 'a+') do |f|
    f.puts msg
    f.flush
  end
end

def flock(file, mode)
  file.flock(File::LOCK_UN)
  success = file.flock(mode)                       # http://www.ruby-doc.org/core-1.8.7/File.html#method-i-flock
  if success
    begin
      yield file
    ensure
      file.flock(File::LOCK_UN)
      file.close
    end
  end
  return success
end

#FIXED: тут было File.join( ".", DAEMON_LOCK ) что неверно, мы не обязательно в "." работаем
d_lock = File.open(File.join(Evented::LOGS_DIR, Evented::DAEMON_LOCK), 'r+')
if d_lock.flock(File::LOCK_SH|File::LOCK_NB) != 0
  d_lock.close
  exit 1
end

flock(d_lock, File::LOCK_EX) do
  log "*********** Starting daemon on dir #{Evented::LOGS_DIR} on file #{@events_log_path} code_version #{Evented::CODE_VERSION} **********"
  loop do
    if (@idle & ((Time.now - @idle_start).to_i >= (IDLE_TIMEOUT * 60)))  then terminate_self("idle timeout") end
    manage_events_log()
    sleep 10
  end
end
