require "sharpeye"

require 'Triangulator'
include System::Collections::Generic

class Serdce2Loader < Loader

  #   feel(p) is invoked to determine - can this file load data or not?
  #
  #   p.ext // file extension in lower case, e.g. "dat"
  #   p.first_2k // first 2048 bytes of file data.
  #   p.first_lines // ascii presentation of first2k. If file contains \0, firstlines is cutted to that first \0.
  #   p.name // file name without extension
  #   p.path // full file path

  def self.feel(p) # Файл определяем по наличию "X Y Z" в первой строке
    p.first_lines.split(/\n/)[0].length == 73 and p.ext == "txt" ? 1.1 : 0.0
    # =~ /X\s+Y\s+Z/ ? 2.5 : 0.0
  end

  #   load(p) is invoked to load data, if this script is winner of feelings
  def load(p)
    @p = p
    @model  = add_model( "#{p.name} serdce" )
    #mis = add_model( "#{p.name} mishca" )
    #mis.color_str = "red"
    #mis.back_color_str = ""

    file = File.new( p.path , "r")

    n = 0
    nvec = 100
    merids = 10
    @gammas = {}
    @datas = []

    while ((pos = file.pos) and (line = file.gets))
      @datas[n] = line.unpack("a8a8a8a8a8a8a8a8a8").map{ |e| e.to_f } # gamma,psi,fi,x,y,z,vx,vy,vz
#      next if @datas[n] [1] != 0.001
      gamma = @datas[n][ 0 ]
      @gammas[ gamma ] ||= []
      @gammas[ gamma ] << n
      n += 1
      puts "line #{n}" if n%1000 == 0
    end
    @gamma_keys = @gammas.keys.sort

    add_slider( @model, "Sphere radius",0,10,1 ) { |v| @sphere_radius = v / 10.0; show_gamma_many }
    @sphere_radius = 0.0 #0.10
    
    add_slider( @model, "Line radius",0,40,1 ) { |v| @line_radius = v / 40.0; show_gamma_many }
    @line_radius = 0.10 / 4.0
    
    add_checkbox( @model, "Show surfaces", true ) { |v| @show_surface = v; show_gamma_many }
    @show_surface = true

    @sliders = []
    for i in (1..5) do 
      @sliders << add_slider( @model, "Gamma slice #{i}", 0, @gamma_keys.length, 0 ) { |v| show_gamma_many }
    end
    @sliders[0].value = 1
    show_gamma_many

    file.close

    puts "Well done"
    puts "Got gammas total #{@gammas.keys.length} = #{@gammas.keys.sort}"
  end

  def add_checkbox( model,title,checked=1 )
    c = System::Windows::Controls::CheckBox.new  
    c.is_checked = checked ? true : false
    c.checked do |sender, args|
      yield true
    end
    c.unchecked do |sender, args|
      yield false
    end    

    model.add_ui_element(c,title)
    c
  end
  

  def add_slider( model,title,min,max,value=1 )
    slider = System::Windows::Controls::Slider.new
    slider.minimum = min
    slider.maximum = max
    slider.value = value
    slider.IsSnapToTickEnabled = true
    slider.TickFrequency = 1
    slider.TickPlacement = TickPlacement.Both
    slider.AutoToolTipPlacement = AutoToolTipPlacement.BottomRight

    slider.value_changed do |sender, args|
       yield slider.value.to_i
    end
    
    model.add_ui_element(slider,title)
    slider  
  end
  
  def show_gamma( gamma_index,reset_model = true )
    return nil if gamma_index <= 0
    gamma = @gamma_keys[ gamma_index-1 ]
    indices = @gammas[ gamma ]
    if indices.nil?
      puts "Data with gamma=#{gamma} not found!"
      return nil
    end
    @model.reset( true ) if reset_model
    xx,yy,zz = nil,nil,nil

    show_lines = @line_radius > 0
    show_spheres = @sphere_radius > 0 and (show_lines and @line_radius < @sphere_radius)
    
    show_surface = @show_surface

    vertices_for_tri = List.of(Triangulator::Geometry::Point).new

    for i in indices do
      gamma,psi,fi,x,y,z,vx,vy,vz = @datas[ i ]

      if show_spheres
        @model.add_sphere( x, y, z, @sphere_radius, 3, 2 )
      end

      if show_lines and xx 
        @model.add_line( Point3D.new(x, y, z), Point3D.new( xx,yy,zz ),@line_radius, 6 ) #r / 4.0
      end

      if show_surface
        vertices_for_tri << Triangulator::Geometry::Point.new(x,y,z)
      end
      
      xx,yy,zz = x,y,z
    end

    if show_surface
      cache_name = "serdce-#{@p.path}-#{@p.time}-gamma-#{gamma}"
      tris = scene.cache_read( cache_name )
      if tris.nil?
        puts "Triangulating #{vertices_for_tri.count} nodes for gamma=#{gamma} ##start"
        tris = Triangulator::Delauney::Triangulate(vertices_for_tri)
        puts "We got #{tris.count} triangles ##stop"    
        scene.cache_write( cache_name,tris)
      else
        puts "CACHE HIT for gamma=#{gamma}"
      end
      
      puts "Adding points and triangles to model ##start"
      t0 = @model.mesh.positions.count
      m2 = @model
      vertices_for_tri.each{ |v| m2.add_node( v.X, v.Y, v.Z ) }
      tris.each{ |t| m2.add_triangle_trusted( t0+t.p2, t0+t.p1, t0+t.p3 ) }      
      puts "Points and triangles added ##stop"
    end

    @model.refresh if reset_model
    puts "Gamma=#{gamma} | #{indices.length} points ##title"
    gamma
  end

  def show_gamma_many
    @model.reset( true )
    gg = []
    for sl in @sliders do
      gg << show_gamma( sl.value.to_i, false )
    end
    @model.refresh
    puts "Gammas: #{gg.compact.sort} ##title"
  end

end