require "#{File.expand_path(File.dirname(__FILE__))}/serdce_data_loader"

class SerdceAllPoints < SerdceDataLoader

  def self.feel(p) 
    super(p) ? 1.0 : 0.0
  end

  def load(p)
    load_data(p)
    @p = p

    @model  = add_model( "#{p.name} points" )

    add_slider( @model, "Sphere radius",0,10,1 ) { |v| @sphere_radius = v / 10.0; render }
    @sphere_radius = 0.1 #0.10

    render
  end

  def render
    @model.reset
    n = 0
    for p in @datas
      gamma,psi,fi,x,y,z,vx,vy,vz = p
      @model.add_sphere( x, y, z, @sphere_radius, 3, 2 )
      n = n+1
      puts "Adding point #{n}" if n%1000 == 0
    end
    @model.refresh
  end

  def foo

    @global = scene.add_model_group( "#{p.name} gamma slices" )
    @groups = []
    @surfaces = []
    @lines = []
    @balls = []
    for i in (1..5) do 
      @groups << scene.add_model_group( "Gamma N#{i}",@global )

      @balls << scene.add_model( "Points N#{i}",nil,true,@groups.last )
      @lines << scene.add_model( "Lines N#{i}",nil,true,@groups.last )
      @surfaces << scene.add_model( "Surface N#{i}",nil,true,@groups.last )
    end

    add_slider( @model, "Sphere radius",0,10,1 ) { |v| @sphere_radius = v / 10.0; show_gamma_many }
    @sphere_radius = 0.0 #0.10
    
    add_slider( @model, "Line radius",0,40, 2 ) { |v| @line_radius = v / 40.0; show_gamma_many }
    @line_radius = 2 * 0.10 / 4.0
    
    add_checkbox( @model, "Show surfaces", true ) { |v| @show_surface = v; show_gamma_many }
    @show_surface = true

    @sliders = []
    for i in (1..5) do 
      @sliders << add_slider( @model, "Gamma slice #{i}", 0, @gamma_keys.length, 0 ) { |v| show_gamma_many }
    end
    @sliders[0].value = 1

    show_gamma_many

    puts "Well done"
    # puts "Got gammas total #{@gammas.keys.length} = #{@gammas.keys.sort}"
  end

  
  def show_gamma( gamma_index,reset_model = true,model_index = 0 )
    return nil if gamma_index <= 0

    gamma = @gamma_keys[ gamma_index-1 ]
    indices = @gammas[ gamma ]
    if indices.nil?
      puts "Data with gamma=#{gamma} not found!"
      return nil
    end

    ball_model = @balls[ model_index ]
    lines_model = @lines[ model_index ]
    surf_model = @surfaces[ model_index ]

    [ball_model,lines_model,surf_model].each{ |m| m.reset }

    xx,yy,zz = nil,nil,nil

    show_lines = @line_radius > 0
    show_spheres = @sphere_radius > 0 and (show_lines and @line_radius < @sphere_radius)
    show_surface = @show_surface
#    puts "Going to render: #{inspect_it :show_lines,:show_spheres,:show_surface}
    puts "Going to render: show_lines=#{show_lines}, show_spheres = #{show_spheres}, show_surface=#{show_surface}"

    vertices_for_tri = List.of(Triangulator::Geometry::Point).new

    for i in indices do
      gamma,psi,fi,x,y,z,vx,vy,vz = @datas[ i ]

      if show_spheres
        ball_model.add_sphere( x, y, z, @sphere_radius, 3, 2 )
      end

      if show_lines and xx and z == zz
        lines_model.add_line( Point3D.new(x, y, z), Point3D.new( xx,yy,zz ),@line_radius, 6 ) #r / 4.0
      end

      if show_surface and z == zz
        vertices_for_tri << Triangulator::Geometry::Point.new(x,y,z)
      end
      
      xx,yy,zz = x,y,z
    end

    if show_surface
      cache_name = "serdce-#{@p.path}-#{@p.time}-gamma-#{gamma}-v8-#{@psi_zero}" ## #{File.mtime(__FILE__)}"
      tris = scene.cache_read( cache_name )
      if tris.nil?
        puts "Triangulating #{vertices_for_tri.count} nodes for gamma=#{gamma} ##start"
        tris = Triangulator::Delauney::Triangulate(vertices_for_tri)
        puts "We got #{tris.count} triangles ##stop"
        scene.cache_write( cache_name,tris)
      else
        puts "CACHE HIT for gamma=#{gamma}"
      end
      
      puts "Adding points and triangles to model ##start"
      m2 = surf_model

      debug = false
      if debug
        @armodel ||= add_model("arrows debug")
        @armodel.reset(true)
        tris.each{ |t|
          r = 0.1
          @armodel.add_arrow( vertices_for_tri[ t.p2 ].to_point, vertices_for_tri[ t.p1 ].to_point,r )
          @armodel.add_arrow( vertices_for_tri[ t.p1 ].to_point, vertices_for_tri[ t.p3 ].to_point,r )
          @armodel.add_arrow( vertices_for_tri[ t.p3 ].to_point, vertices_for_tri[ t.p2 ].to_point,r )
        }
        @armodel.refresh
      end

      begin
        t0 = m2.mesh.positions.count
        vertices_for_tri.each{ |v| m2.add_node( v.X, v.Y, v.Z ) }
        #tris.each{ |t| m2.add_triangle( t0+t.p2, t0+t.p1, t0+t.p3 ) }
        tris.each{ |t| m2.add_triangle_trusted( t0+t.p2, t0+t.p1, t0+t.p3 ) }
      end


      puts "Points and triangles added ##stop"
    end

    [ball_model,lines_model,surf_model].each{ |m| m.refresh }

    puts "Gamma=#{gamma} | #{indices.length} points ##title"
    gamma
  end

  def show_gamma_many
    gg = []
    @sliders.each_with_index do |sl,i|
      gg << show_gamma( sl.value.to_i, true, i )
    end
    puts "Gammas: #{gg.compact.sort} ##title"
  end

end